@@ -1,8 +1,13 @@
-__version__ = '0.1.3'
+"""
+Core Hyperpython API
+"""
+__version__ = '0.2.0'
 __author__ = 'Fábio Macêdo Mendes'
 
+# We put all
 try:
-    from .core import Element, Text
+    from .core import Element, Text, Block
+    from .render import render, render_html
     from .tags import (
         HTML5, h,
         body, head, meta, link, title, div, span, article, aside, details, footer,
@@ -17,5 +22,8 @@ try:
         time, var, output, acronym, applet, big, basefont, center, dir_, font,
         frame, frameset, noframes, strike, tt
     )
-except:
-    pass
\ No newline at end of file
+    from .utils import escape, unescape, safe, sanitize
+    from .helpers import classes
+except Exception as exc:
+    print('Caught exception:', exc)
+    print('This is expected during installation.')
