@@ -1,12 +1,14 @@
 import io
-from types import MappingProxyType
+from collections import Sequence
 
 from markupsafe import Markup
+from types import MappingProxyType
 
+from .helpers import classes
 from .renderers import dump_attrs, render_pretty
-from .utils import flatten, unescape, escape as _escape
+from .utils import unescape, escape as _escape
 
-SEQUENCE_TYPES = (tuple, list, type(x for x in []))
+SEQUENCE_TYPES = (tuple, list, type(x for x in []), type(map(lambda: 0, [])))
 
 
 class ElementMixin:
@@ -20,15 +22,30 @@ class ElementMixin:
     classes = property(lambda self: self.attrs.get('class', []))
     id = property(lambda self: self.attrs.get('id'))
 
+    @id.setter
+    def id(self, value):
+        self.attrs['id'] = value
+
     children = ()
     requires = ()
 
     is_element = False
     is_void = False
 
-    def render(self, **kwargs):
+    def _repr_html_(self):
+        return self.__html__()
+
+    def render(self):
         """
-        Render element as string.
+        Renders object as string.
+        """
+        file = io.StringIO()
+        self.dump(file)
+        return file.getvalue()
+
+    def dump(self, file):
+        """
+        Dump contents of element in the given file.
         """
         raise NotImplementedError
 
@@ -73,6 +90,20 @@ class ElementMixin:
                 yield obj
                 yield from (child.walk_tags() for child in obj.children)
 
+    def add_child(self, value):
+        """
+        Add child element to data structure.
+
+        Caveat: Hyperpython *do not* enforce immutability, but it is a good
+        practice to keep HTML data structures immutable.
+        """
+        try:
+            append = self.children.append
+        except AttributeError:
+            raise TypeError('cannot change immutable structure')
+        else:
+            append(as_child(value))
+
 
 # ------------------------------------------------------------------------------
 class Element(ElementMixin):
@@ -102,12 +133,12 @@ class Element(ElementMixin):
             raise ValueError('void elements cannot define children')
 
         if isinstance(item, SEQUENCE_TYPES):
-            children = flatten(item)
+            self.children.extend(map(as_child, item))
         elif item is None:
-            children = []
+            pass
         else:
-            children = [item]
-        return Element(self.tag, self.attrs, children, False, self.requires)
+            self.children.append(as_child(item))
+        return self
 
     def __str__(self):
         return str(self.__html__())
@@ -155,14 +186,6 @@ class Element(ElementMixin):
                 child.dump(file)
         write(f'</{self.tag}>')
 
-    def render(self):
-        """
-        Renders object as string.
-        """
-        file = io.StringIO()
-        self.dump(file)
-        return file.getvalue()
-
     def json(self):
         """
         JSON-compatible representation of object.
@@ -186,16 +209,31 @@ class Element(ElementMixin):
         new.requires = self.requires
         return new
 
-    def add_class(self, *classes):
+    def add_class(self, cls, first=False):
         """
-        Add classes to the class list.
-
-        Does nothing if class is already present.
+        Add class or group of classes to the class list.
         """
+        new_classes = classes(cls)
         try:
-            self.attrs['class'].extend(classes)
+            old_classes = self.attrs['class']
         except KeyError:
-            self.attrs['class'] = list(classes)
+            self.attrs['class'] = new_classes
+        else:
+            if first:
+                class_set = set(new_classes)
+                new_classes.extend(x for x in old_classes if x not in class_set)
+                self.attrs['class'][:] = new_classes
+            else:
+                class_set = set(old_classes)
+                old_classes.extend(x for x in new_classes if x not in class_set)
+        return self
+
+    def set_class(self, cls):
+        """
+        Replace all current classes by the new ones.
+        """
+        self.attrs['class'] = list(classes(cls))
+        return self
 
 
 # ------------------------------------------------------------------------------
@@ -203,7 +241,6 @@ class Text(Markup, ElementMixin):
     """
     It extends the Markup object with a Element-compatible API.
     """
-
     id = None
     classes = property(lambda self: [])
     unescaped = property(unescape)
@@ -245,20 +282,53 @@ class Text(Markup, ElementMixin):
         return {'text': str(self)} if self.escape else {'raw': str(self)}
 
 
+class Block(ElementMixin, Sequence):
+    """
+    Represents a list of elements *not* wrapped in a tag.
+    """
+
+    id = None
+    classes = property(lambda self: [])
+
+    def __init__(self, children, requires=()):
+        self.children = list(map(as_child, children))
+        self.requires = list(requires)
+
+    def __iter__(self):
+        return iter(self.children)
+
+    def __getitem__(self, idx):
+        return self.children[idx]
+
+    def __len__(self):
+        return len(self.children)
+
+    def dump(self, file):
+        for child in self.children:
+            child.dump(file)
+
+    def copy(self):
+        return Block(list(self.children), requires=list(self.requires))
+
+    def json(self):
+        return {'body': [x.to_json() for x in self.children]}
+
+
 #
 # Helper functions
 #
 def as_attr(name, value):
     """
-    Convert arbitrary object to a valid value for an Element attribute.
+    Enforces an arbitrary pair of attribute name and value has a compatible
+    Hyperpython values.
 
     Args:
         name: attribute name
         value: attribute value
     """
     if name == 'class':
-        if isinstance(value, str):
-            return name, value.split(' ')
+        value = list(classes(value))
+        return name, value
     return name, value
 
 
@@ -266,6 +336,7 @@ def as_child(value):
     """
     Convert arbitrary object to a compatible Element object.
     """
+
     if isinstance(value, (Element, Text)):
         return value
     elif isinstance(value, (str, Markup)):
@@ -275,8 +346,13 @@ def as_child(value):
     elif isinstance(value, Tag):
         return Tag._h_function(value.tag)
     elif hasattr(value, '__html__'):
-        return Text(Markup(value.__html__()), escape=False)
+        return Text(value.__html__(), escape=False)
+    elif hasattr(value, '__hyperpython__'):
+        return value.__hyperpython__()
     else:
+        data = str(value)
+        if data == value:
+            return Text(data)
         type_name = value.__class__.__name__
         raise TypeError('invalid type for a child node: %s' % type_name)
 
