@@ -1,4 +1,4 @@
 from .attrs import dump_attrs, render_attrs
-from .helpers import join_classes, js_class, render_pretty
+from .helpers import render_pretty
 from .html import dump_html, render_html
 from .single_attr import dump_single_attr, render_single_attr
